#!/usr/bin/perl

use strict;
use warnings;

sub POE::Kernel::USE_SIGCHLD () { 1 }
sub POE::Kernel::ASSERT_USAGE () { 1 }
sub POE::Kernel::TRACE_SIGNALS () { 0 }

use Test::More;

use POE;
use POEx::Clock;
use POSIX qw( SIGALRM SIGTERM );

sub DEBUG () { 1 }

my $interval = 10;

POEx::Clock->spawn({ interval=>$interval, 
                     debug=>DEBUG, 
                     signal=>SIGALRM 
                  });

my $COUNT;
POE::Session->create( 
        inline_states => {
            _start => sub {
                $poe_kernel->sig( 'clock-skew' => 'skew' );
                $poe_kernel->alias_set( 'test' );
                diag( "Sleep ", $interval );
                $poe_kernel->delay( first => $interval );
            },
            _stop => sub {
                DEBUG and warn "stop";
            },
            first => sub {
                my $ses = $poe_kernel->alias_resolve( 'clock-skew' );
                my $child = $ses->[2]{skew}[0]{child};
                isa_ok( $child, 'POE::Component::Generic' );
                # Kill the child.  The child should respawn by itself
                $child->kill( SIGTERM );
                diag( "Sleep ", $interval*1.5 );

                $poe_kernel->delay( after => $interval*1.5 );
            },
            # we detect the respawned child by faking a clock skew
            after => sub {
    
                my $ses = $poe_kernel->alias_resolve( 'clock-skew' );
                my $child = $ses->[2]{skew}[0]{child};
                isa_ok( $child, 'POE::Component::Generic' );

                $COUNT = 0;
                # We can't adjust the system clock.  But we can fake a clock skew 
                # by sending RTMIN early.  This is going to provoke 2 clock-skew events.
                $child->kill( SIGALRM );
                diag( "Sleep ", $interval*1.5 );
                $poe_kernel->delay( done => $interval*1.5 );
            },
            skew => sub {
                return unless defined $COUNT;
                my( $signal, $epsilon, $exepected ) = @_[ ARG0..$#_ ];
                $COUNT++;
                ok( abs($epsilon)<10, "Detected a fake skew" );
                
            },
            done => sub {
                diag( "DONE" );
                $poe_kernel->signal( $poe_kernel, 'shutdown' );
            }
        }
    );

$poe_kernel->run;
pass( "Exited" );

ok( $COUNT, "Detected a fake skew after child exited" );

# We can't use an exact test count; we could have a real clock skew during testing.
done_testing();
